<?php

namespace WPIS;

$path = $_SERVER['DOCUMENT_ROOT'];

include_once $path . '/wp-config.php';
include_once $path . '/wp-load.php';
include_once $path . '/wp-includes/class-wpdb.php';
include_once $path . '/wp-includes/pluggable.php';

include_once __DIR__ . '/../core/SyncManager.php';

/**
 * Class ApiAdmin
 *
 * This class handles the AJAX exhanges with the administration
 */

 class ApiAdmin {

    private $apiUrl = WPIS::API_URL . '/wordpress/';

    private $authToken = null;
    private $payload = [];

    static public function init() : ApiAdmin {
        $class = new ApiAdmin();

        $class->wp_db = $GLOBALS['wpdb'];
        $class->wp_table_connexion = $class->wp_db->prefix . 'wpis_connexion';

        $class->payload = json_decode(file_get_contents('php://input'));
        $class->payload = $class->payload ? $class->payload : [];

        $public = false;
        $prefixPublic = 'wp_ajax_nopriv_wpis_';
        $prefixPrivate = 'wp_ajax_wpis_';
        $prefix = $public ? $prefixPublic : $prefixPrivate;

        add_action( $prefix . 'login', [$class, 'login'] );
        add_action( $prefix . 'checkStatus', [$class, 'checkStatus'] );
        add_action( $prefix . 'checkAccess', [$class, 'checkAccess'] );
        add_action( $prefix . 'activateAccess', [$class, 'activateAccess'] );
        add_action( $prefix . 'removeAccess', [$class, 'removeAccess'] );
        add_action( $prefix . 'updateSettings', [$class, 'updateSettings'] );
        add_action( $prefix . 'loadSettings', [$class, 'loadSettings'] );
        add_action( $prefix . 'importChecking', [$class, 'importChecking'] );
        add_action( $prefix . 'importNext', [$class, 'importNext'] );
        add_action( $prefix . 'updateStart', [$class, 'updateStart'] );
        add_action( $prefix . 'updateProgress', [$class, 'updateProgress'] );
        add_action( $prefix . 'removeAllEstates', [$class, 'removeAllEstates'] );
        add_action( $prefix . 'removeInactiveEstates', [$class, 'removeInactiveEstates'] );
        add_action( $prefixPublic . 'importNext', [$class, 'importNext'] );

        return $class;
    }

    private function post($endpoint, $payload = [], $headers = []) {

        $params = [
            'wpis_version' => get_plugin_data(__DIR__ . '/../wp-immo-sync.php')['Version'],
            'wp_version'   => get_bloginfo('version'),
            'referer'      => $_SERVER['HTTP_HOST']
        ];

        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL => $this->apiUrl . $endpoint . '?' . http_build_query($params),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => http_build_query($payload),
            CURLOPT_HTTPHEADER => $headers,
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        curl_close($curl);
        if ($err) {
          echo "cURL Error #:" . $err;
        }
        else {
            return (object) ['code' => $httpcode, 'data' => $response];
        }
    }

    private function response( $code, $data = false ) : void {
        http_response_code($code);
        header('Content-Type: application/json');
        echo json_encode($data);
        exit;
    }

    public function checkStatus(){

        $access = $this->wp_db->get_results("SELECT access_id, name, account FROM $this->wp_table_connexion LIMIT 1");

        $response = [
            'debug' => SettingsManager::get('debug', false),
            'access' => $access ? $access[0] : false
        ];

        return $this->response(200, $response);
    }

    public function login(){

        $response = $this->post('login', $this->payload, [
            "Content-Type: application/x-www-form-urlencoded",
            "cache-control: no-cache"
        ]);

        if( $response->code !== 200 ){
            return $this->response($response->code, ['success' => false, 'message' => $response->data]);
        }

        $this->response(200, ['success' => true, 'data' => json_decode($response->data)]);
    }

    public function checkAccess(){

        if( !$this->payload->token ){
            return $this->response(400, ['message' => 'Unauthenticated.']);
        }

        $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
        $response = $this->post('check-access', ['domain' => $protocol . $_SERVER['HTTP_HOST']], [
            'Content-Type: application/x-www-form-urlencoded',
            'Authorization: Bearer ' . $this->payload->token
        ]);

        $data = json_decode($response->data);

        if( $response->code !== 200 ){
            $message = isset($data->message) ? ['code' => $response->code, 'message' => $data->message] : ['code' => $response->code, 'message' => 'N/A'];
            return $this->response($response->code, $message);
        }

        if( empty($data) ){
            return $this->response(200, (Object)[]);
        }
        $this->response(200, $data);
    }

    public function activateAccess(){

        if( !$this->payload->token ){
            return $this->response(400, ['message' => 'Unauthenticated.']);
        }

        // print_r($this->payload);exit;
        if( !$this->payload->accessId ){
            return $this->response(400, ['message' => 'Access Id is required.']);
        }

        $wpisAcccess = $this->wp_db->get_results("SELECT * FROM $this->wp_table_connexion LIMIT 1");

        if( !empty($wpisAcccess) ){
            return $this->response(400, ['message' => 'Access has been already registred']);
        }

        $response = $this->post('activate-access', ['accessId' => $this->payload->accessId], [
            'Content-Type: application/x-www-form-urlencoded',
            'Authorization: Bearer ' . $this->payload->token
        ]);

        if( $response->code !== 200 ){
            $data = json_decode($response->data);
            return $this->response($response->code, ['message' => $data->message]);
        }

        $data = json_decode($response->data);
        $data = [
            'access_id' => $data->id,
            'name' => $data->name,
            'account' => $data->account,
            'token' => $data->token
        ];

        if( !$this->wp_db->insert($this->wp_table_connexion, $data) ) {
            return $this->response(500, 'An error occured while registrating access in database.');
        }

        $this->response(200, ['message' => 'Access registred']);
    }

    public function removeAccess(){

        if( !$this->payload->accessId ){
            return $this->response(400, ['message' => 'Access Id is required.']);
        }

        $this->wp_db->delete($this->wp_table_connexion, ['access_id' => $this->payload->accessId]);

        $this->response(200, ['message' => 'Access removed.']);
    }

    public function loadSettings(){

        $settings = SettingsManager::getAll();

        $postTypes = get_post_types();

        unset($postTypes['attachment']);
        unset($postTypes['page']);
        unset($postTypes['revision']);
        unset($postTypes['nav_menu_item']);
        unset($postTypes['custom_css']);
        unset($postTypes['customize_changeset']);
        unset($postTypes['oembed_cache']);
        unset($postTypes['wp_block']);
        unset($postTypes['user_request']);
        unset($postTypes['wpis_estates']);

        $cron = [];

        foreach($settings as $key => $setting){
            if( $setting->name === 'enableCrons' && $setting->value === '1'){
                $cron['url'] = get_site_url(null, '/wp-content/plugins/wp-immo-sync/core/CronManager.php');
            }
        }
        foreach($settings as $key => $setting){
            if( isset($cron['url']) && $setting->name === 'cronKey' ){
                $cron['key'] = $setting->value;
            }
            if( $setting->name === 'cronKey' ){
                unset($settings[$key]);
            }
        }

        $response = [
            'debug'         => SettingsManager::get('debug', false),
            'settings'      => $settings,
            'cron'          => $cron,
            'configuration' => [
                'max_execution_time' => [
                    'current'  => ini_get('max_execution_time'),
                    'required' => '600'
                ]
            ],
            'formData' => [
                'pageList'      => get_pages(),
                'postTypesList' => $postTypes
            ]
        ];

        $this->response(200, $response);
    }

    public function updateSettings(){

        $acceptedKeys = [
            'accentColor',
            'estatePageId',
            'enableImportSystem',
            'enableTaxonomies',
            'autoRemoveUpdatedEstates',
            'enableImportCustomPostType',
            'customPostType',
            'enableCrons',
            'enableCronImport',
            'enableCronUpdate',
            'enableCronRemove',
            'enableProjectManagement',
            'enableProjectManagementCustomPostType',
            'projectCustomPostType',
            'enableAgentManagement',
            'enableAgentManagementCustomPostType',
            'agentCustomPostType',
            'enableVueShortcodes'
        ];

        foreach( $this->payload as $key => $value ){
            if( !in_array($key, $acceptedKeys) ){
                $this->response(422, ['success' => false, 'message' => 'Unprocessable Entity.']);
            }

            if( $key === 'enableCrons' && $value === '1'){
                $token = $this->generateRandomString('12');
                if( is_null( SettingsManager::get('cronKey') ) ){
                    SettingsManager::set('cronKey', $token);
                }
            }

            SettingsManager::set($key, $value);
        }

        $this->response(200, ['success' => true]);
    }

    public function importNext(){
        $response = SyncManager::init()->next($this->payload);
        $this->response(200, $response);
    }

    public function importChecking(){
        $response = SyncManager::init()->checking();
        $this->response(200, $response);
    }

    public function updateStart(){
        $response = SyncManager::init()->updateStart($this->payload);
        $this->response(200, $response);
    }

    public function updateProgress(){
        $response = SyncManager::init()->updateProgress($this->payload);
        $this->response(200, $response);
    }

    public function removeInactiveEstates(){
        $response = SyncManager::init()->removeInactives();
        $this->response(200, $response);
    }

    public function removeAllEstates(){
        $response = SyncManager::init()->removeAll();
        $this->response(200, $response);
    }

    public function logError($error = []){
        $response = $this->post('log', ['accessId' => $this->payload->accessId, 'domain' => $protocol . $_SERVER['HTTP_HOST'], 'error' => $error], [
            'Content-Type: application/x-www-form-urlencoded',
            'Authorization: Bearer ' . $this->payload->token
        ]);
    }

    private function generateRandomString($length = 10) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

}