<?php
/**
 * Plugin Name: WP Immo Sync
 * Text Domain: wpis
 * Domain Path: /languages
 * Plugin URI: https://wp.immosync.com
 * Description: Import and synchronize your estates in Wordpress with your realestate software (Whise, Omnicasa, Sweepbright, Activimmo)
 * Version: 1.8.9
 * Author: Jeremy Jamar
 * Author URI: https://www.immosync.com/
 * License: GPL2
 */

namespace WPIS;

require_once 'core/EstateProperties.php';
require_once 'core/AdminAssetsManager.php';
require_once 'core/ClientAssetsManager.php';
require_once 'core/PostTypesManager.php';
require_once 'core/ShortcodesManager.php';
require_once 'utilities/SettingsManager.php';
require_once 'utilities/Logger.php';
require_once 'utilities/Updater.php';
require_once 'utilities/Tools.php';
require_once 'api/AdminApi.php';
require_once 'api/PublicApi.php';
require_once 'api/ImmosyncClient.php';
require_once 'core/Filters/NumberFilter.php';
require_once 'core/Filters/DecimalFilter.php';
require_once 'core/Filters/PriceFilter.php';
require_once 'core/Filters/CapitalizeFilter.php';
require_once 'core/Filters/UppercaseFilter.php';
require_once 'core/Filters/EpcImageFilter.php';
require_once 'core/Filters/YesNoFilter.php';

require_once ABSPATH . 'wp-admin/includes/upgrade.php';

class WPIS {

    public const API_URL = 'https://api.immosync.com';

    public function __construct(){

        AdminAssetsManager::init();
        ClientAssetsManager::init();
        PostTypesManager::init();
        ShortcodesManager::init();
        ApiAdmin::init();
        ApiPublic::init();

        $this->wp_db = $GLOBALS['wpdb'];
        $this->wp_table_connexion = $this->wp_db->prefix . 'wpis_connexion';
        $this->wp_table_settings = $this->wp_db->prefix . 'wpis_settings';

        register_activation_hook( __FILE__ , [$this, 'wpis_install']);
        register_deactivation_hook( __FILE__, [$this, 'wpis_uninstall']);
        add_action( 'plugins_loaded', [$this, 'wpis_loadTextDomain'] );
    }

    public function wpis_install() {

        $charset_collate = $this->wp_db->get_charset_collate();

        $create_table_connexion = "CREATE TABLE IF NOT EXISTS $this->wp_table_connexion (
          access_id tinytext NOT NULL,
          name tinytext NOT NULL,
          account tinytext NOT NULL,
          token TEXT NOT NULL,
          activated_at TIMESTAMP NOT NULL
        ) $charset_collate;";
        $this->wp_db->query( $create_table_connexion );

        $create_table_settings = "CREATE TABLE IF NOT EXISTS $this->wp_table_settings (
          name VARCHAR(50) UNIQUE NOT NULL,
          value VARCHAR(50),
          updated_at TIMESTAMP NOT NULL
        ) $charset_collate;";
        $this->wp_db->query( $create_table_settings );
    }

    public function wpis_uninstall() {
        $sql = 'DROP TABLE IF EXISTS ' . $this->wp_table_connexion . ';';
        $this->wp_db->query($sql);
        $sql = 'DROP TABLE IF EXISTS ' . $this->wp_table_settings . ';';
        $this->wp_db->query($sql);
    }

    public function wpis_loadTextDomain() {
        load_plugin_textdomain( 'wpis', false, basename( dirname( __FILE__ ) ) . '/languages' );
    }
}

new WPIS();